import 'dart:developer';

import 'package:hub/shared/helpers/database/database_helper.dart';
import 'package:hub/shared/helpers/storage/base_storage.dart';
import 'package:sqflite/sqflite.dart';

abstract class LicenseLocalDataSource {
  Future<void> init();
  Future<void> setByKey(final List<String> key, final String display);
  Future<bool> isNewVersion();
  Future<String?> g(String key);
  Future<void> s<T>(String key, T value);
  Future<void> d(String key);
  Future<void> c();
}

class LicenseLocalDataSourceImpl implements LicenseLocalDataSource {
  static final LicenseLocalDataSourceImpl _instance = LicenseLocalDataSourceImpl._internal();
  factory LicenseLocalDataSourceImpl() => _instance;
  LicenseLocalDataSourceImpl._internal();

  static const String tableLicense = 'license';

  static String get createLicenseTable => '''
      CREATE TABLE IF NOT EXISTS $tableLicense (
        key TEXT UNIQUE,
        display TEXT,
        expirationDate TEXT,
        startDate TEXT,
        quantity TEXT
      );
    ''';

  static String get deleteLicenseTable => 'DROP TABLE IF EXISTS $tableLicense;';

  static String get updatePetsHistoryTrigger => '''
      CREATE TRIGGER update_fre_hub_pets_history
      AFTER INSERT ON $tableLicense
      WHEN NEW.key = 'FRE-HUB-PETS'
      BEGIN
        INSERT OR REPLACE INTO $tableLicense (key, display, expirationDate, startDate, quantity)
        VALUES ('FRE-HUB-PETS-HISTORY', NEW.display, NEW.expirationDate, NEW.startDate, NEW.quantity);
      END;
    ''';

  static String get dropPetsHistoryTrigger => 'DROP TRIGGER IF EXISTS update_fre_hub_pets_history;';

  static String get updateDisplayTrigger => '''
      CREATE TRIGGER update_display_trigger
      AFTER UPDATE ON $tableLicense
      WHEN NEW.key IN ('FRE-HUB-OPENED-VISITS', 'FRE-HUB-VEHICLES', 'FRE-HUB-RESIDENTS', 'FRE-HUB-PETS')
      BEGIN
        UPDATE $tableLicense
        SET display = CASE
          WHEN (SELECT COUNT(*) FROM $tableLicense WHERE key IN ('FRE-HUB-OPENED-VISITS', 'FRE-HUB-VEHICLES', 'FRE-HUB-RESIDENTS', 'FRE-HUB-PETS') AND display = 'VISIVEL') > 0
          THEN 'VISIVEL'
          ELSE 'INVISIVEL'
        END
        WHERE key = 'FRE-HUB-ABOUT-PROPERTY';
      END;
    ''';

  static String get dropDisplayTrigger => 'DROP TRIGGER IF EXISTS update_display_trigger;';

  Future<void> init() async {
    await DatabaseStorage.instance.init();
  }

  Future<void> setByKey(final List<String?> key, final String display) async {
    if (key.isEmpty) return;
    for (var element in key) {
      if (element != null) {
        log('setByKey($element, $display)');
        await s(element, {
          'display': display,
          'expirationDate': '',
          'startDate': '',
          'quantity': '',
        });
      }
    }
  }

  Future<bool> isNewVersion() async {
    var response = await DatabaseStorage.database.query(tableLicense, where: 'key = ?', whereArgs: [KeychainStorageKey.isNewVersion.value], columns: ['display']);
    if (response.isEmpty) {
      return false;
    }
    log('isNewVersion(): ${response.first['display']}');
    return response.first['display'] == 'true';
  }

  Future<String?> g(String key) async {
    var response = await DatabaseStorage.database.query(tableLicense, where: 'key = ?', whereArgs: [key]);
    if (response.isEmpty) {
      return null;
    }
    return response.first.toString();
  }

  Future<void> s<T>(String key, T value) async {
    log('setLicense($key, $value)');
    value as Map<String, dynamic>;

    await DatabaseStorage.database.insert(
        tableLicense,
        {
          'key': key,
          'display': value['display'],
          'expirationDate': value['expirationDate'],
          'startDate': value['startDate'],
          'quantity': value['quantity'],
        },
        conflictAlgorithm: ConflictAlgorithm.replace);
    return;
  }

  Future<void> d(String key) async {
    await DatabaseStorage.database.delete(tableLicense, where: 'key = ?', whereArgs: [key]);
  }

  Future<void> c() async {
    await DatabaseStorage.database.delete(tableLicense);
  }
}
